@@ -1,5 +1,5 @@
 /*
- * iperf, Copyright (c) 2014-2020, 2023, The Regents of the University of
+ * iperf, Copyright (c) 2014-2020, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -31,8 +31,9 @@
 
 #include <sys/time.h>
 #include <sys/types.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#include <inttypes.h>
+#endif
 #include <sys/select.h>
 #include <sys/socket.h>
 #ifndef _GNU_SOURCE
@@ -50,101 +51,84 @@
 #include <sys/cpuset.h>
 #endif /* HAVE_CPUSET_SETAFFINITY */
 
+#if defined(HAVE_INTTYPES_H)
+# include <inttypes.h>
+#else
+# ifndef PRIu64
+#  if sizeof(long) == 8
+#   define PRIu64		"lu"
+#  else
+#   define PRIu64		"llu"
+#  endif
+# endif
+#endif
+
 #include "timer.h"
 #include "queue.h"
 #include "cjson.h"
 #include "iperf_time.h"
-#include "portable_endian.h"
 
 #if defined(HAVE_SSL)
 #include <openssl/bio.h>
 #include <openssl/evp.h>
 #endif // HAVE_SSL
 
-#include "iperf_pthread.h"
-
-/*
- * Atomic types highly desired, but if not, we approximate what we need
- * with normal integers and warn.
- */
-#ifdef HAVE_STDATOMIC_H
-#include <stdatomic.h>
-#else
-#warning "No <stdatomic.h> available."
-typedef uint64_t atomic_uint_fast64_t;
-#endif // HAVE_STDATOMIC_H
-
 #if !defined(__IPERF_API_H)
-typedef uint_fast64_t iperf_size_t;
-typedef atomic_uint_fast64_t atomic_iperf_size_t;
+typedef uint64_t iperf_size_t;
 #endif // __IPERF_API_H
 
-#if (defined(__vxworks)) || (defined(__VXWORKS__))
-typedef unsigned int uint
-#endif // __vxworks or __VXWORKS__
-
-struct iperf_sctp_info
-{
-    long rtt;
-    long pmtu;
-    uint32_t wnd;
-    uint32_t cwnd;
-};
-
 struct iperf_interval_results
 {
-    atomic_iperf_size_t bytes_transferred; /* bytes transferred in this interval */
+    iperf_size_t bytes_transferred; /* bytes transfered in this interval */
     struct iperf_time interval_start_time;
     struct iperf_time interval_end_time;
     float     interval_duration;
 
     /* for UDP */
-    int64_t   interval_packet_count;
-    int64_t   interval_outoforder_packets;
-    int64_t   interval_cnt_error;
-    int64_t   packet_count;
+    int       interval_packet_count;
+    int       interval_outoforder_packets;
+    int       interval_cnt_error;
+    int       packet_count;
     double    jitter;
-    int64_t   outoforder_packets;
-    int64_t   cnt_error;
+    int       outoforder_packets;
+    int       cnt_error;
 
     int omitted;
-#if (defined(linux) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)) && \
+#if (defined(linux) || defined(__FreeBSD__) || defined(__NetBSD__)) && \
 	defined(TCP_INFO)
-    struct tcp_info tcpInfo; /* getsockopt(TCP_INFO) for Linux, {Free,Net,Open}BSD */
+    struct tcp_info tcpInfo; /* getsockopt(TCP_INFO) for Linux, {Free,Net}BSD */
 #else
     /* Just placeholders, never accessed. */
     char *tcpInfo;
 #endif
-#if defined(HAVE_SCTP_H)
-    struct iperf_sctp_info sctp_info;
-#endif /* HAVE_SCTP_H */
-    long interval_retrans;
-    long snd_cwnd;
-    long snd_wnd;
+    int interval_retrans;
+    int interval_sacks;
+    int snd_cwnd;
+    int snd_wnd;
     TAILQ_ENTRY(iperf_interval_results) irlistentries;
     void     *custom_data;
-    long rtt;
-    long rttvar;
-    long pmtu;
-    long reorder;
+    int rtt;
+    int rttvar;
+    int pmtu;
 };
 
 struct iperf_stream_result
 {
-    atomic_iperf_size_t bytes_received;
-    atomic_iperf_size_t bytes_sent;
-    atomic_iperf_size_t bytes_received_this_interval;
-    atomic_iperf_size_t bytes_sent_this_interval;
-    atomic_iperf_size_t bytes_sent_omit;
-    long stream_prev_total_retrans;
-    long stream_retrans;
-    long stream_reorder;
-    long stream_max_rtt;
-    long stream_min_rtt;
-    long stream_sum_rtt;
+    iperf_size_t bytes_received;
+    iperf_size_t bytes_sent;
+    iperf_size_t bytes_received_this_interval;
+    iperf_size_t bytes_sent_this_interval;
+    iperf_size_t bytes_sent_omit;
+    int stream_prev_total_retrans;
+    int stream_retrans;
+    int stream_prev_total_sacks;
+    int stream_sacks;
+    int stream_max_rtt;
+    int stream_min_rtt;
+    int stream_sum_rtt;
     int stream_count_rtt;
-    long stream_max_snd_cwnd;
-    long stream_max_snd_wnd;
+    int stream_max_snd_cwnd;
+    int stream_max_snd_wnd;
     struct iperf_time start_time;
     struct iperf_time end_time;
     struct iperf_time start_time_fixed;
@@ -162,7 +146,7 @@ struct iperf_settings
     int       blksize;              /* size of read/writes (-l) */
     iperf_size_t  rate;                 /* target data rate for application pacing*/
     iperf_size_t  bitrate_limit;   /* server's maximum allowed total data rate for all streams*/
-    double        bitrate_limit_interval;  /* interval for averaging total data rate */
+    double        bitrate_limit_interval;  /* interval for avaraging total data rate */
     int           bitrate_limit_stats_per_interval;     /* calculated number of stats periods for averaging total data rate */
     uint64_t  fqrate;               /* target data rate for FQ pacing*/
     int	      pacing_timer;	    /* pacing timer in microseconds */
@@ -182,15 +166,9 @@ struct iperf_settings
     char      *client_password;
     EVP_PKEY  *client_rsa_pubkey;
 #endif // HAVE_SSL
-    int       skip_rx_copy;         /* Whether to ignore received messages data, using MSG_TRUNC option */
     int	      connect_timeout;	    /* socket connection timeout, in ms */
     int       idle_timeout;         /* server idle time timeout */
-    unsigned int snd_timeout; /* Timeout for sending tcp messages in active mode, in us */
     struct iperf_time rcv_timeout;  /* Timeout for receiving messages in active mode, in us */
-    int       cntl_ka;              /* Use Control TCP connection Keepalive */
-    int       cntl_ka_keepidle;     /* Control TCP connection Keepalive idle time (TCP_KEEPIDLE) */
-    int       cntl_ka_interval;     /* Control TCP connection Keepalive interval between retries (TCP_KEEPINTV) */
-    int       cntl_ka_count;        /* Control TCP connection Keepalive number of retries (TCP_KEEPCNT) */
 };
 
 struct iperf_test;
@@ -199,17 +177,13 @@ struct iperf_stream
 {
     struct iperf_test* test;
 
-    pthread_t thr;
-    int thread_created;
-    int       done;
-
     /* configurable members */
     int       local_port;
     int       remote_port;
     int       socket;
     int       id;
     int       sender;
-	/* XXX: is settings just a pointer to the same struct in iperf_test? if not,
+	/* XXX: is settings just a pointer to the same struct in iperf_test? if not, 
 		should it be? */
     struct iperf_settings *settings;	/* pointer to structure settings */
 
@@ -227,16 +201,15 @@ struct iperf_stream
      * for udp measurements - This can be a structure outside stream, and
      * stream can have a pointer to this
      */
-    int64_t   packet_count;
-    int64_t   peer_packet_count;
-    int64_t   peer_omitted_packet_count;
-    int64_t   omitted_packet_count;
+    int       packet_count;
+    int	      peer_packet_count;
+    int       omitted_packet_count;
     double    jitter;
     double    prev_transit;
-    int64_t   outoforder_packets;
-    int64_t   omitted_outoforder_packets;
-    int64_t   cnt_error;
-    int64_t   omitted_cnt_error;
+    int       outoforder_packets;
+    int       omitted_outoforder_packets;
+    int       cnt_error;
+    int       omitted_cnt_error;
     uint64_t  target;
 
     struct sockaddr_storage local_addr;
@@ -284,19 +257,8 @@ enum iperf_mode {
 	BIDIRECTIONAL = -1
 };
 
-enum debug_level {
-    DEBUG_LEVEL_ERROR = 1,
-    DEBUG_LEVEL_WARN = 2,
-    DEBUG_LEVEL_INFO = 3,
-    DEBUG_LEVEL_DEBUG = 4,
-    DEBUG_LEVEL_MAX = 4
-};
-
-
 struct iperf_test
 {
-    pthread_mutex_t print_mutex;
-
     char      role;                             /* 'c' lient or 's' erver */
     enum iperf_mode mode;
     int       sender_has_retransmits;
@@ -312,7 +274,6 @@ struct iperf_test
     int       server_port;
     int       omit;                             /* duration of omit period (-O flag) */
     int       duration;                         /* total duration of test (-t flag) */
-    int       max_server_duration;               /* maximum possible duration of test as enforced by the server (--max-server-duration flag) */
     char     *diskfile_name;			/* -F option */
     int       affinity, server_affinity;	/* -A option */
 #if defined(HAVE_CPUSET_SETAFFINITY)
@@ -329,7 +290,6 @@ struct iperf_test
     FILE     *outfile;
 
     int       ctrl_sck;
-    int       mapped_v4;
     int       listener;
     int       prot_listener;
 
@@ -339,7 +299,6 @@ struct iperf_test
     char      *server_authorized_users;
     EVP_PKEY  *server_rsa_private_key;
     int       server_skew_threshold;
-    int       use_pkcs1_padding;
 #endif // HAVE_SSL
 
     /* boolean variables for Options */
@@ -350,13 +309,8 @@ struct iperf_test
     int       bidirectional;                    /* --bidirectional */
     int	      verbose;                          /* -V option - verbose mode */
     int	      json_output;                      /* -J option - JSON output */
-    int	      json_stream;                      /* --json-stream */
-    int       json_stream_full_output;          /* --json-stream-full-output */
-    void      (*json_callback) (struct iperf_test *, char *); /* allow user apps to receive the
-                                                JSON strings,instead of writing them to the output file */
     int	      zerocopy;                         /* -Z option - use sendfile */
     int       debug;				/* -d option - enable debug */
-    enum      debug_level debug_level;          /* -d option option - level of debug messages to show */
     int	      get_server_output;		/* --get-server-output */
     int	      udp_counters_64bit;		/* --use-64-bit-udp-counters */
     int       forceflush; /* --forceflush - flushing output at every interval */
@@ -364,7 +318,6 @@ struct iperf_test
     int	      repeating_payload;                /* --repeating-payload */
     int       timestamps;			/* --timestamps */
     char     *timestamp_format;
-    int       mptcp;				/* -m, --mptcp */
 
     char     *json_output_string; /* rendered JSON output if json_output is set */
     /* Select related parameters */
@@ -372,7 +325,7 @@ struct iperf_test
     fd_set    read_set;                         /* set of read sockets */
     fd_set    write_set;                        /* set of write sockets */
 
-    /* Interval related members */
+    /* Interval related members */ 
     int       omitting;
     double    stats_interval;
     double    reporter_interval;
@@ -389,15 +342,15 @@ struct iperf_test
 
     int       num_streams;                      /* total streams in the test (-P) */
 
-    atomic_iperf_size_t bytes_sent;
-    atomic_iperf_size_t blocks_sent;
+    iperf_size_t bytes_sent;
+    iperf_size_t blocks_sent;
 
-    atomic_iperf_size_t bytes_received;
-    atomic_iperf_size_t blocks_received;
+    iperf_size_t bytes_received;
+    iperf_size_t blocks_received;
 
     iperf_size_t bitrate_limit_stats_count;               /* Number of stats periods accumulated for server's total bitrate average */
     iperf_size_t *bitrate_limit_intervals_traffic_bytes;  /* Pointer to a cyclic array that includes the last interval's bytes transferred */
-    iperf_size_t bitrate_limit_last_interval_index;       /* Index of the last interval traffic inserted into the cyclic array */
+    iperf_size_t bitrate_limit_last_interval_index;       /* Index of the last interval traffic insrted into the cyclic array */
     int          bitrate_limit_exceeded;                  /* Set by callback routine when average data rate exceeded the server's bitrate limit */
 
     int server_last_run_rc;                      /* Save last server run rc for next test */
@@ -449,8 +402,6 @@ struct iperf_test
 
 #define UDP_BUFFER_EXTRA 1024
 
-#define MAX_PARAMS_JSON_STRING 8 * 1024
-
 /* constants for command line arg sanity checks */
 #define MB (1024 * 1024)
 #define MAX_TCP_BUFFER (512 * MB)
@@ -462,7 +413,6 @@ struct iperf_test
 #define MIN_INTERVAL 0.1
 #define MAX_INTERVAL 60.0
 #define MAX_TIME 86400
-#define MAX_OMIT_TIME 600
 #define MAX_BURST 1000
 #define MAX_MSS (9 * 1024)
 #define MAX_STREAMS 128
@@ -471,19 +421,4 @@ struct iperf_test
 
 extern int gerror; /* error value from getaddrinfo(3), for use in internal error handling */
 
-/* UDP "connect" message and reply (textual value for Wireshark, etc. readability - legacy was numeric) */
-
-#if BYTE_ORDER == BIG_ENDIAN
-#define UDP_CONNECT_MSG 0x39383736
-#define UDP_CONNECT_REPLY 0x36373839
-#define LEGACY_UDP_CONNECT_REPLY 0xb168de3a
-#else
-#define UDP_CONNECT_MSG 0x36373839          // "6789" - legacy value was 123456789
-#define UDP_CONNECT_REPLY 0x39383736        // "9876" - legacy value was 987654321
-#define LEGACY_UDP_CONNECT_REPLY 987654321  // Old servers may still reply with the legacy value
-#endif
-
-/* In Reverse mode, maximum number of packets to wait for "accept" response - to handle out of order packets */
-#define MAX_REVERSE_OUT_OF_ORDER_PACKETS 2
-
 #endif /* !__IPERF_H */
