@@ -39,6 +39,10 @@
 #include <sys/select.h>
 #include <limits.h>
 
+#ifdef HAVE_NETINET_SCTP_H
+#include <netinet/sctp.h>
+#endif /* HAVE_NETINET_SCTP_H */
+
 #include "iperf.h"
 #include "iperf_api.h"
 #include "iperf_sctp.h"
@@ -56,7 +60,7 @@ iperf_sctp_recv(struct iperf_stream *sp)
 #if defined(HAVE_SCTP_H)
     int r;
 
-    r = Nread_no_select(sp->socket, sp->buffer, sp->settings->blksize, Psctp);
+    r = Nread(sp->socket, sp->buffer, sp->settings->blksize, Psctp);
     if (r < 0)
         return r;
 
@@ -78,7 +82,7 @@ iperf_sctp_recv(struct iperf_stream *sp)
 }
 
 
-/* iperf_sctp_send
+/* iperf_sctp_send 
  *
  * sends the data for SCTP
  */
@@ -90,7 +94,7 @@ iperf_sctp_send(struct iperf_stream *sp)
 
     r = Nwrite(sp->socket, sp->buffer, sp->settings->blksize, Psctp);
     if (r < 0)
-        return r;
+        return r;    
 
     sp->result->bytes_sent += r;
     sp->result->bytes_sent_this_interval += r;
@@ -161,8 +165,7 @@ iperf_sctp_listen(struct iperf_test *test)
     int s, opt, saved_errno;
 
     close(test->listener);
-    test->listener = -1;
-
+   
     snprintf(portstr, 6, "%d", test->server_port);
     memset(&hints, 0, sizeof(hints));
     /*
@@ -224,13 +227,13 @@ iperf_sctp_listen(struct iperf_test *test)
     }
 
 #if defined(IPV6_V6ONLY) && !defined(__OpenBSD__)
-    if (res->ai_family == AF_INET6 && (test->settings->domain == AF_UNSPEC ||
+    if (res->ai_family == AF_INET6 && (test->settings->domain == AF_UNSPEC || 
         test->settings->domain == AF_INET6)) {
         if (test->settings->domain == AF_UNSPEC)
             opt = 0;
         else
             opt = 1;
-        if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY,
+        if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, 
 		       (char *) &opt, sizeof(opt)) < 0) {
 	    saved_errno = errno;
 	    close(s);
@@ -277,7 +280,7 @@ iperf_sctp_listen(struct iperf_test *test)
     }
 
     test->listener = s;
-
+  
     return s;
 #else
     i_errno = IENOSCTP;
@@ -730,49 +733,3 @@ out:
     return -1;
 #endif /* HAVE_SCTP_H */
 }
-
-
-/* iperf_sctp_get_rtt
- *
- * Get SCTP stream RTT.
- * Assuming that iperf3 supports only one-toone SCTP associassion, and not one-to-many associassion.
- * 
- * Main resouses used are RFC-6458, man pages for SCTP,
- * https://docs.oracle.com/cd/E19253-01/817-4415/sockets-199/index.html.
- * 
- */
-int
-iperf_sctp_get_info(struct iperf_stream *sp, struct iperf_sctp_info *sctp_info)
-{
-#if defined(HAVE_SCTP_H)
-    struct sctp_status status;
-    socklen_t len;
-    sctp_assoc_t assoc_id;
-    int rc = 0;
-
-    if (sp->test->protocol->id != Psctp) {
-        rc = -1;
-    } else {
-#ifdef SCTP_FUTURE_ASSOC
-        assoc_id = SCTP_FUTURE_ASSOC;
-#else
-	assoc_id = 0;
-#endif
-        len = sizeof(status);
-        rc = sctp_opt_info(sp->socket, assoc_id, SCTP_STATUS, &status, &len);
-        if (rc < 0) {
-            if (sp->test->debug_level >= DEBUG_LEVEL_ERROR)
-                iperf_err(sp->test, "sctp_opt_info get SCTP_STATUS for socket %d failed with errno %d - %s", sp->socket, errno, strerror(errno));
-        } else {
-            sctp_info->wnd = status.sstat_rwnd;
-            sctp_info->rtt = status.sstat_primary.spinfo_srtt;
-            sctp_info->pmtu = status.sstat_primary.spinfo_mtu;
-            sctp_info->cwnd = status.sstat_primary.spinfo_cwnd;
-        }
-    }
-
-    return rc;
-#else
-    return -1;
-#endif /* HAVE_SCTP_H */
-}
