@@ -1,5 +1,5 @@
 /*
- * iperf, Copyright (c) 2014-2023, The Regents of the University of
+ * iperf, Copyright (c) 2014-2019, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -65,9 +65,6 @@
 #include "net.h"
 #include "timer.h"
 
-static int nread_read_timeout = 10;
-static int nread_overall_timeout = 30;
-
 /*
  * Declaration of gerror in iperf_error.c.  Most other files in iperf3 can get this
  * by including "iperf.h", but net.c lives "below" this layer.  Clearly the
@@ -122,13 +119,12 @@ timeout_connect(int s, const struct sockaddr *name, socklen_t namelen,
  * Copyright: http://swtch.com/libtask/COPYRIGHT
 */
 
-/* create a socket */
+/* make connection to server */
 int
-create_socket(int domain, int proto, const char *local, const char *bind_dev, int local_port, const char *server, int port, struct addrinfo **server_res_out)
+netdial(int domain, int proto, const char *local, const char *bind_dev, int local_port, const char *server, int port, int timeout)
 {
     struct addrinfo hints, *local_res = NULL, *server_res = NULL;
     int s, saved_errno;
-    char portstr[6];
 
     if (local) {
         memset(&hints, 0, sizeof(hints));
@@ -141,12 +137,8 @@ create_socket(int domain, int proto, const char *local, const char *bind_dev, in
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = domain;
     hints.ai_socktype = proto;
-    snprintf(portstr, sizeof(portstr), "%d", port);
-    if ((gerror = getaddrinfo(server, portstr, &hints, &server_res)) != 0) {
-	if (local)
-	    freeaddrinfo(local_res);
+    if ((gerror = getaddrinfo(server, NULL, &hints, &server_res)) != 0)
         return -1;
-    }
 
     s = socket(server_res->ai_family, proto, 0);
     if (s < 0) {
@@ -212,8 +204,6 @@ create_socket(int domain, int proto, const char *local, const char *bind_dev, in
 	}
 	/* Unknown protocol */
 	else {
-	    close(s);
-	    freeaddrinfo(server_res);
 	    errno = EAFNOSUPPORT;
             return -1;
 	}
@@ -227,22 +217,7 @@ create_socket(int domain, int proto, const char *local, const char *bind_dev, in
         }
     }
 
-    *server_res_out = server_res;
-    return s;
-}
-
-/* make connection to server */
-int
-netdial(int domain, int proto, const char *local, const char *bind_dev, int local_port, const char *server, int port, int timeout)
-{
-    struct addrinfo *server_res = NULL;
-    int s, saved_errno;
-
-    s = create_socket(domain, proto, local, bind_dev, local_port, server, port, &server_res);
-    if (s < 0) {
-      return -1;
-    }
-
+    ((struct sockaddr_in *) server_res->ai_addr)->sin_port = htons(port);
     if (timeout_connect(s, (struct sockaddr *) server_res->ai_addr, server_res->ai_addrlen, timeout) < 0 && errno != EINPROGRESS) {
 	saved_errno = errno;
 	close(s);
@@ -266,7 +241,7 @@ netannounce(int domain, int proto, const char *local, const char *bind_dev, int
 
     snprintf(portstr, 6, "%d", port);
     memset(&hints, 0, sizeof(hints));
-    /*
+    /* 
      * If binding to the wildcard address with no explicit address
      * family specified, then force us to get an AF_INET6 socket.  On
      * CentOS 6 and MacOS, getaddrinfo(3) with AF_UNSPEC in ai_family,
@@ -287,7 +262,7 @@ netannounce(int domain, int proto, const char *local, const char *bind_dev, int
     hints.ai_socktype = proto;
     hints.ai_flags = AI_PASSIVE;
     if ((gerror = getaddrinfo(local, portstr, &hints, &res)) != 0)
-        return -1;
+        return -1; 
 
     s = socket(res->ai_family, proto, 0);
     if (s < 0) {
@@ -310,7 +285,7 @@ netannounce(int domain, int proto, const char *local, const char *bind_dev, int
     }
 
     opt = 1;
-    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
+    if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, 
 		   (char *) &opt, sizeof(opt)) < 0) {
 	saved_errno = errno;
 	close(s);
@@ -332,7 +307,7 @@ netannounce(int domain, int proto, const char *local, const char *bind_dev, int
 	    opt = 0;
 	else
 	    opt = 1;
-	if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY,
+	if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, 
 		       (char *) &opt, sizeof(opt)) < 0) {
 	    saved_errno = errno;
 	    close(s);
@@ -352,7 +327,7 @@ netannounce(int domain, int proto, const char *local, const char *bind_dev, int
     }
 
     freeaddrinfo(res);
-
+    
     if (proto == SOCK_STREAM) {
         if (listen(s, INT_MAX) < 0) {
 	    saved_errno = errno;
@@ -375,37 +350,10 @@ Nread(int fd, char *buf, size_t count, int prot)
 {
     register ssize_t r;
     register size_t nleft = count;
-    struct iperf_time ftimeout = { 0, 0 };
-
-    fd_set rfdset;
-    struct timeval timeout = { nread_read_timeout, 0 };
-
-    /*
-     * fd might not be ready for reading on entry. Check for this
-     * (with timeout) first.
-     *
-     * This check could go inside the while() loop below, except we're
-     * currently considering whether it might make sense to support a
-     * codepath that bypassese this check, for situations where we
-     * already know that fd has data on it (for example if we'd gotten
-     * to here as the result of a select() call.
-     */
-    {
-        FD_ZERO(&rfdset);
-        FD_SET(fd, &rfdset);
-        r = select(fd + 1, &rfdset, NULL, NULL, &timeout);
-        if (r < 0) {
-            return NET_HARDERROR;
-        }
-        if (r == 0) {
-            return 0;
-        }
-    }
 
     while (nleft > 0) {
         r = read(fd, buf, nleft);
         if (r < 0) {
-            /* XXX EWOULDBLOCK can't happen without non-blocking sockets */
             if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)
                 break;
             else
@@ -415,66 +363,6 @@ Nread(int fd, char *buf, size_t count, int prot)
 
         nleft -= r;
         buf += r;
-
-        /*
-         * We need some more bytes but don't want to wait around
-         * forever for them. In the case of partial results, we need
-         * to be able to read some bytes every nread_timeout seconds.
-         */
-        if (nleft > 0) {
-            struct iperf_time now;
-
-            /*
-             * Also, we have an approximate upper limit for the total time
-             * that a Nread call is supposed to take. We trade off accuracy
-             * of this timeout for a hopefully lower performance impact.
-             */
-            iperf_time_now(&now);
-            if (ftimeout.secs == 0) {
-                ftimeout = now;
-                iperf_time_add_usecs(&ftimeout, nread_overall_timeout * 1000000L);
-            }
-            if (iperf_time_compare(&ftimeout, &now) < 0) {
-                break;
-            }
-
-            FD_ZERO(&rfdset);
-            FD_SET(fd, &rfdset);
-            r = select(fd + 1, &rfdset, NULL, NULL, &timeout);
-            if (r < 0) {
-                return NET_HARDERROR;
-            }
-            if (r == 0) {
-                break;
-            }
-        }
-    }
-    return count - nleft;
-}
-
-/********************************************************************/
-/* reads 'count' bytes from a socket - but without using select()   */
-/********************************************************************/
-int
-Nread_no_select(int fd, char *buf, size_t count, int prot)
-{
-    register ssize_t r;
-    register size_t nleft = count;
-
-    while (nleft > 0) {
-        r = read(fd, buf, nleft);
-        if (r < 0) {
-            /* XXX EWOULDBLOCK can't happen without non-blocking sockets */
-            if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)
-                break;
-            else
-                return NET_HARDERROR;
-        } else if (r == 0)
-            break;
-
-        nleft -= r;
-        buf += r;
-
     }
     return count - nleft;
 }
@@ -497,17 +385,14 @@ Nwrite(int fd, const char *buf, size_t count, int prot)
 		case EINTR:
 		case EAGAIN:
 #if (EAGAIN != EWOULDBLOCK)
-                    /* XXX EWOULDBLOCK can't happen without non-blocking sockets */
 		case EWOULDBLOCK:
 #endif
-		if (count == nleft)
-		    return NET_SOFTERROR;
 		return count - nleft;
 
-                case ENOBUFS :
-                return NET_SOFTERROR;
+		case ENOBUFS:
+		return NET_SOFTERROR;
 
-                default:
+		default:
 		return NET_HARDERROR;
 	    }
 	} else if (r == 0)
@@ -538,8 +423,8 @@ has_sendfile(void)
 int
 Nsendfile(int fromfd, int tofd, const char *buf, size_t count)
 {
-#if defined(HAVE_SENDFILE)
     off_t offset;
+#if defined(HAVE_SENDFILE)
 #if defined(__FreeBSD__) || (defined(__APPLE__) && defined(__MACH__) && defined(MAC_OS_X_VERSION_10_6))
     off_t sent;
 #endif
@@ -570,7 +455,6 @@ Nsendfile(int fromfd, int tofd, const char *buf, size_t count)
 		case EINTR:
 		case EAGAIN:
 #if (EAGAIN != EWOULDBLOCK)
-                    /* XXX EWOULDBLOCK can't happen without non-blocking sockets */
 		case EWOULDBLOCK:
 #endif
 		if (count == nleft)
